@@ -26,7 +26,8 @@ from .c2pa import (
     Signer,
     Stream,
     sdk_version,
-    read_ingredient_file
+    read_ingredient_file,
+    load_settings
 )  # NOQA
 
 # Re-export C2paError and its subclasses
@@ -39,5 +40,6 @@ __all__ = [
     'Signer',
     'Stream',
     'sdk_version',
-    'read_ingredient_file'
+    'read_ingredient_file',
+    'load_settings'
 ]
