@@ -10,59 +10,53 @@ import ctypes
 import logging
 import platform
 from pathlib import Path
-from typing import Optional, Tuple
+from typing import Optional
 from enum import Enum
 
 # Debug flag for library loading
 DEBUG_LIBRARY_LOADING = False
 
-# Configure logging
-logging.basicConfig(
-    level=logging.INFO,
-    format='%(asctime)s - %(levelname)s - %(message)s',
-    force=True  # Force configuration even if already configured
-)
-logger = logging.getLogger(__name__)
+# Create a module-specific logger with NullHandler
+# to avoid interfering with global configuration
+logger = logging.getLogger("c2pa.loader")
+logger.addHandler(logging.NullHandler())
 
 
 class CPUArchitecture(Enum):
     """CPU architecture enum for platform-specific identifiers."""
     AARCH64 = "aarch64"
     X86_64 = "x86_64"
+    ARM64 = "arm64"
 
 
-def get_platform_identifier(cpu_arch: Optional[CPUArchitecture] = None) -> str:
-    """Get the full platform identifier (arch-os) for the current system,
+def get_platform_identifier() -> str:
+    """Get the platform identifier (arch-os) for the current system,
     matching the downloaded identifiers used by the Github publisher.
 
-    Args:
-        cpu_arch: Optional CPU architecture for macOS.
-        If not provided, returns universal build.
-        Only used on macOS systems.
-
     Returns one of:
-    - universal-apple-darwin (for Mac, when cpu_arch is None)
+    - universal-apple-darwin (for Mac universal)
     - aarch64-apple-darwin (for Mac ARM64)
     - x86_64-apple-darwin (for Mac x86_64)
     - x86_64-pc-windows-msvc (for Windows 64-bit)
     - x86_64-unknown-linux-gnu (for Linux 64-bit)
+    - aarch64-unknown-linux-gnu (for Linux ARM)
     """
     system = platform.system().lower()
 
     if system == "darwin":
-        if cpu_arch is None:
-            return "universal-apple-darwin"
-        elif cpu_arch == CPUArchitecture.AARCH64:
+        # Prefer specific architecture over universal for better performance
+        # Universal libraries will be attempted as fallback if specific ones aren't found
+        current_arch = _get_architecture()
+        if current_arch == CPUArchitecture.ARM64.value:
             return "aarch64-apple-darwin"
-        elif cpu_arch == CPUArchitecture.X86_64:
+        elif current_arch == CPUArchitecture.X86_64.value:
             return "x86_64-apple-darwin"
         else:
-            raise ValueError(
-                f"Unsupported CPU architecture for macOS: {cpu_arch}")
+            return "universal-apple-darwin"
     elif system == "windows":
         return "x86_64-pc-windows-msvc"
     elif system == "linux":
-        if _get_architecture() in ['arm64', 'aarch64']:
+        if _get_architecture() in [CPUArchitecture.ARM64.value, CPUArchitecture.AARCH64.value]:
             return "aarch64-unknown-linux-gnu"
         return "x86_64-unknown-linux-gnu"
     else:
@@ -79,13 +73,14 @@ def _get_architecture() -> str:
     if sys.platform == "darwin":
         # On macOS, we need to check if we're running under Rosetta
         if platform.processor() == 'arm':
-            return 'arm64'
+            return CPUArchitecture.ARM64.value
         else:
-            return 'x86_64'
+            return CPUArchitecture.X86_64.value
     elif sys.platform == "linux":
         return platform.machine()
     elif sys.platform == "win32":
-        # win32 will cover all Windows versions (the 32 is a historical quirk)
+        # win32 will cover all Windows versions
+        # (the 32 is a historical quirk)
         return platform.machine()
     else:
         raise RuntimeError(f"Unsupported platform: {sys.platform}")
@@ -103,14 +98,15 @@ def _get_platform_dir() -> str:
     elif sys.platform == "linux":
         return "unknown-linux-gnu"
     elif sys.platform == "win32":
-        # win32 will cover all Windows versions (the 32 is a historical quirk)
+        # win32 will cover all Windows versions
+        # (the 32 is a historical quirk)
         return "pc-windows-msvc"
     else:
         raise RuntimeError(f"Unsupported platform: {sys.platform}")
 
 
 def _load_single_library(lib_name: str,
-                         search_paths: list[Path]) -> Optional[ctypes.CDLL]:
+                         search_paths: list[Path]) -> tuple[Optional[ctypes.CDLL], list[tuple[Path, Exception]]]:
     """
     Load a single library from the given search paths.
 
@@ -119,33 +115,38 @@ def _load_single_library(lib_name: str,
         search_paths: List of paths to search for the library
 
     Returns:
-        The loaded library or None if loading failed
+        A tuple of (loaded library or None, list of (path, error) for files that were found but failed to load)
     """
-    if DEBUG_LIBRARY_LOADING:
+    if DEBUG_LIBRARY_LOADING:  # pragma: no cover
         logger.info(f"Searching for library '{lib_name}' in paths: {[str(p) for p in search_paths]}")
     current_arch = _get_architecture()
-    if DEBUG_LIBRARY_LOADING:
+    if DEBUG_LIBRARY_LOADING:  # pragma: no cover
         logger.info(f"Current architecture: {current_arch}")
 
+    load_errors = []
+
     for path in search_paths:
         lib_path = path / lib_name
-        if DEBUG_LIBRARY_LOADING:
+        if DEBUG_LIBRARY_LOADING:  # pragma: no cover
             logger.info(f"Checking path: {lib_path}")
         if lib_path.exists():
-            if DEBUG_LIBRARY_LOADING:
+            if DEBUG_LIBRARY_LOADING:  # pragma: no cover
                 logger.info(f"Found library at: {lib_path}")
             try:
-                return ctypes.CDLL(str(lib_path))
+                return ctypes.CDLL(str(lib_path)), []
             except Exception as e:
                 error_msg = str(e)
+                load_errors.append((lib_path, e))
                 if "incompatible architecture" in error_msg:
                     logger.error(f"Architecture mismatch: Library at {lib_path} is not compatible with current architecture {current_arch}")
                     logger.error(f"Error details: {error_msg}")
+                elif "GLIBC" in error_msg or "version" in error_msg.lower():
+                    logger.error(f"Library dependency error at {lib_path}: {error_msg}")
                 else:
                     logger.error(f"Failed to load library from {lib_path}: {e}")
         else:
             logger.debug(f"Library not found at: {lib_path}")
-    return None
+    return None, load_errors
 
 
 def _get_possible_search_paths() -> list[Path]:
@@ -159,7 +160,7 @@ def _get_possible_search_paths() -> list[Path]:
     platform_dir = _get_platform_dir()
     platform_id = get_platform_identifier()
 
-    if DEBUG_LIBRARY_LOADING:
+    if DEBUG_LIBRARY_LOADING:  # pragma: no cover
         logger.info(f"Using platform directory: {platform_dir}")
         logger.info(f"Using platform identifier: {platform_id}")
 
@@ -187,6 +188,11 @@ def _get_possible_search_paths() -> list[Path]:
         # Add platform identifier subfolder
         possible_paths.append(base_path / platform_id)
 
+    # Add universal fallback for macOS if not already looking for universal
+    if sys.platform == "darwin" and platform_id != "universal-apple-darwin":
+        for base_path in base_paths:
+            possible_paths.append(base_path / "universal-apple-darwin")
+
     # Add system library paths
     possible_paths.extend([Path(p) for p in os.environ.get(
         "LD_LIBRARY_PATH", "").split(os.pathsep) if p])
@@ -200,9 +206,12 @@ def dynamically_load_library(
     Load the dynamic library containing the C-API based on the platform.
 
     Args:
-        lib_name: Optional specific library name to load. If provided, only this library will be loaded.
-        This enables to potentially load wrapper libraries of the C-API that may have an other name
-        (the presence of required symbols will nevertheless be verified once the library is loaded).
+        lib_name: Optional specific library name to load.
+          If provided, only this library will be loaded.
+        This enables to potentially load wrapper libraries
+        of the C-API that may have an other name
+        (the presence of required symbols will nevertheless
+        be verified once the library is loaded).
 
     Returns:
         The loaded library or None if loading failed
@@ -216,17 +225,18 @@ def dynamically_load_library(
     else:
         raise RuntimeError(f"Unsupported platform: {sys.platform}")
 
-    if DEBUG_LIBRARY_LOADING:
+    if DEBUG_LIBRARY_LOADING:  # pragma: no cover
         logger.info(f"Current working directory: {Path.cwd()}")
         logger.info(f"Package directory: {Path(__file__).parent}")
         logger.info(f"System architecture: {_get_architecture()}")
+        logger.info(f"Platform identifier: {get_platform_identifier()}")
+        logger.info(f"Platform directory: {_get_platform_dir()}")
 
     # Check for C2PA_LIBRARY_NAME environment variable
     env_lib_name = os.environ.get("C2PA_LIBRARY_NAME")
     if env_lib_name:
-        if DEBUG_LIBRARY_LOADING:
-            logger.info(
-                f"Using library name from env var C2PA_LIBRARY_NAME: {env_lib_name}")
+        if DEBUG_LIBRARY_LOADING:  # pragma: no cover
+            logger.info(f"Using library name from env var C2PA_LIBRARY_NAME: {env_lib_name}")
         try:
             possible_paths = _get_possible_search_paths()
             lib = _load_single_library(env_lib_name, possible_paths)
@@ -238,26 +248,51 @@ def dynamically_load_library(
                 # name fails
         except Exception as e:
             logger.error(f"Failed to load library from C2PA_LIBRARY_NAME: {e}")
-            # Continue with normal loading if environment variable library name
-            # fails
+            # Continue with normal loading if
+            # environment variable library name fails
 
     possible_paths = _get_possible_search_paths()
 
     if lib_name:
         # If specific library name is provided, only load that one
-        lib = _load_single_library(lib_name, possible_paths)
+        lib, load_errors = _load_single_library(lib_name, possible_paths)
         if not lib:
             platform_id = get_platform_identifier()
             current_arch = _get_architecture()
-            logger.error(f"Could not find {lib_name} in any of the search paths: {[str(p) for p in possible_paths]}")
-            logger.error(f"Platform: {platform_id}, Architecture: {current_arch}")
-            raise RuntimeError(f"Could not find {lib_name} in any of the search paths (Platform: {platform_id}, Architecture: {current_arch})")
+
+            if load_errors:
+                # Library files were found but failed to load
+                error_details = "\n".join([f"  - {path}: {error}" for path, error in load_errors])
+                logger.error(f"Found {lib_name} but failed to load it. Errors encountered:")
+                logger.error(error_details)
+                logger.error(f"Platform: {platform_id}, Architecture: {current_arch}")
+                raise RuntimeError(
+                    f"Found {lib_name} at {len(load_errors)} location(s) but failed to load:\n{error_details}\n"
+                    f"Platform: {platform_id}, Architecture: {current_arch}"
+                )
+            else:
+                # Library file was not found in any search path
+                logger.error(f"Could not find {lib_name} in any of the search paths: {[str(p) for p in possible_paths]}")
+                logger.error(f"Platform: {platform_id}, Architecture: {current_arch}")
+                raise RuntimeError(
+                    f"Could not find {lib_name} in any of the search paths (Platform: {platform_id}, Architecture: {current_arch})"
+                )
         return lib
 
     # Default path (no library name provided in the environment)
-    c2pa_lib = _load_single_library(c2pa_lib_name, possible_paths)
+    c2pa_lib, load_errors = _load_single_library(c2pa_lib_name, possible_paths)
     if not c2pa_lib:
-        logger.error(f"Could not find {c2pa_lib_name} in any of the search paths: {[str(p) for p in possible_paths]}")
-        raise RuntimeError(f"Could not find {c2pa_lib_name} in any of the search paths")
+        if load_errors:
+            # Library files were found but failed to load
+            error_details = "\n".join([f"  - {path}: {error}" for path, error in load_errors])
+            logger.error(f"Found {c2pa_lib_name} but failed to load it. Errors encountered:")
+            logger.error(error_details)
+            raise RuntimeError(
+                f"Found {c2pa_lib_name} at {len(load_errors)} location(s) but failed to load:\n{error_details}"
+            )
+        else:
+            # Library file was not found in any search path
+            logger.error(f"Could not find {c2pa_lib_name} in any of the search paths: {[str(p) for p in possible_paths]}")
+            raise RuntimeError(f"Could not find {c2pa_lib_name} in any of the search paths")
 
     return c2pa_lib
